
import xmltodict

DEFAULT_FSSD_XML_CONFIG ="""<?xml version="1.0" encoding="utf-8"?>
<root>
	<fssd:config xmlns:fssd="http://pimodules.com/">
		<fssd:alerts>
			<fssd:email>
				<fssd:enabled>False</fssd:enabled>
				<fssd:username></fssd:username>
				<fssd:sender-email-address></fssd:sender-email-address>
				<fssd:sender-password></fssd:sender-password>
				<fssd:recipient-email-address></fssd:recipient-email-address>
				<fssd:subject-template></fssd:subject-template>
				<fssd:body-template></fssd:body-template>
				<fssd:server></fssd:server>
				<fssd:security></fssd:security>
				<fssd:port>0</fssd:port>
			</fssd:email>
		</fssd:alerts>
	</fssd:config>
</root>
"""


def read_config_xml(xml, default=True):
	try:
		if default:
			dict = xmltodict.parse(DEFAULT_FSSD_XML_CONFIG)
		else:
			with open(xml, "rt") as fi:
				dict = xmltodict.parse(fi)

	except IOError as e:
		dict = xmltodict.parse(DEFAULT_FSSD_XML_CONFIG)

	return dict

def write_config_xml(xmlfile, dict):
	try:
	        with open(xmlfile, "wt") as fo:
	                xmltodict.unparse(dict, fo, pretty=True)
	except IOError as e:
		print("Error writing XML file: ", e)
		return False

	return True






import smtplib
import subprocess
import datetime
import jinja2
import base64
from email.mime.text import MIMEText


def get_host_name():
	try:
		command = "uname -n"
		process = subprocess.Popen(command.split(), stdout=subprocess.PIPE)
		output = process.communicate()[0]
		return output
	except:
		return None


def get_ip_address():
	try:
		command = "hostname -I"
		process = subprocess.Popen(command.split(), stdout=subprocess.PIPE)
		output = process.communicate()[0]
		return output
	except:
		return None


def sendEmail( emailserver, username, port, security, fromAddr, toAddr, b64Password, msgSubjectTemplate, msgBodyTemplate):
	try:
		port = int(port)
	except ValueError:
		port = 0

	try:
		now = datetime.datetime.now()
		host = get_host_name()
		ipaddress = get_ip_address()
	except:
		raise

	password = base64.b64decode(b64Password)

	tVars= { 'now' : now, 'host' : host, 'ipaddress' : ipaddress }

	try:
		templateLoader = jinja2.FileSystemLoader(searchpath="/")
		templateEnv = jinja2.Environment( loader=templateLoader )

		subjectTemplate = templateEnv.get_template(msgSubjectTemplate)
		bodyTemplate = templateEnv.get_template(msgBodyTemplate)

		msgSubject = subjectTemplate.render(tVars)
		msgBody = bodyTemplate.render(tVars)

	except:
		raise

	# Writing the message
	msg = MIMEText(msgBody)
	msg['Subject'] = msgSubject
	msg['From'] = fromAddr
	msg['To'] = toAddr

	# Sending the message
	try:
		if port:
			serverstring = format("%s:%d" % (emailserver, port))
		else:
			serverstring = emailserver

		if security.lower() in ['ssl']:
			server = smtplib.SMTP_SSL(serverstring)
		else:
			server = smtplib.SMTP(serverstring)
			server.starttls()

		server.login(username, password)
		server.sendmail(fromAddr, [toAddr], msg.as_string())
		server.quit()
	except:
		raise

	return True



import setuptools

with open("README.txt", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pimodules", # Replace with your own username
    version='0.1',
    author='Mike Ray',
    author_email='mike.ray@btinternet.com',
    description="A package which contains code to support PiModules products of various kinds.",
    long_description_content_type="text/markdown",
    url='http://pimodules.com',
    packages=['pimodules'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires='>=2.7',
)




import sys
import os
import time
import atexit
import logging
import logging.handlers

from signal import SIGTERM 



class Daemon(object):
	"""
	A forking daemon
	
	Usage: subclass the Daemon class and override the run() method
	"""
	def __init__(self, pidfile, stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
		self.log = logging.getLogger(__name__)
		self.log.setLevel(logging.DEBUG)
		handler = logging.handlers.SysLogHandler(address = '/dev/log')
		formatter = logging.Formatter('%(module)s[%(process)s]: <%(levelname)s>: %(message)s')
		handler.setFormatter(formatter)
		self.log.addHandler(handler)

		self.stdin = stdin
		self.stdout = stdout
		self.stderr = stderr

		self.pidfile = pidfile
	
	def daemonize(self):
		"""
		do the UNIX double-fork magic, see Stevens' "Advanced 
		Programming in the UNIX Environment" for details (ISBN 0201563177)
		http://www.erlenstar.demon.co.uk/unix/faq_2.html#SEC16
		"""

		try: 
			pid = os.fork() 
			if pid > 0:
				# exit first parent
				sys.exit(0) 
		except OSError as e:
			sys.stderr.write("fork #1 failed: %d (%s)\n" % (e.errno, e.strerror))
			sys.exit(1)
	
		# decouple from parent environment
		os.chdir("/") 
		os.setsid() 
		os.umask(0) 
	
		# do second fork
		try: 
			pid = os.fork() 
			if pid > 0:
				# exit from second parent
				sys.exit(0) 
		except OSError as e:
			sys.stderr.write("fork #2 failed: %d (%s)\n" % (e.errno, e.strerror))
			sys.exit(1) 
	
		# redirect standard file descriptors
		sys.stdout.flush()
		sys.stderr.flush()

		si = open(self.stdin, 'r')
		so = open(self.stdout, 'a+')
		se = open(self.stderr, 'a+')

		os.dup2(si.fileno(), sys.stdin.fileno())
		os.dup2(so.fileno(), sys.stdout.fileno())
		os.dup2(se.fileno(), sys.stderr.fileno())
	
		# write pidfile
		atexit.register(self.deletePidFile)
		pid = str(os.getpid())
		file(self.pidfile,'w+').write("%s\n" % pid)
	
	def deletePidFile(self):
		os.remove(self.pidfile)

	def start(self, daemonize=True):
		"""
		Start the daemon
		"""

		self.daemon = daemonize


		if daemonize:
			# Check for a pidfile to see if the daemon is already running
			try:
				pf = open(self.pidfile,'r')
				pid = int(pf.read().strip())
				pf.close()
			except IOError:
				pid = None
	
		if daemonize:
			if pid:
				message = "pidfile %s already exist. Daemon already running?\n"
				sys.stderr.write(message % self.pidfile)
				sys.exit(1)
		
		# Start the daemon
		if daemonize:
			self.daemonize()

		self.run()

	def stop(self):
		"""
		Stop the daemon
		"""

		if self.daemon:
			# Get the pid from the pidfile
			try:
				pf = file(self.pidfile,'r')
				pid = int(pf.read().strip())
				pf.close()
			except IOError:
				print('could not get pid')
				pid = None
	
		if self.daemon:
			if not pid:
				message = "pidfile %s does not exist. Daemon not running?\n"
				sys.stderr.write(message % self.pidfile)
				return # not an error in a restart

		if self.daemon:
			# Try killing the daemon process	
			try:
				while 1:
					os.kill(pid, SIGTERM)
					time.sleep(0.1)
			except OSError as err:
				err = str(err)
				if err.find("No such process") > 0:
					if os.path.exists(self.pidfile):
						os.remove(self.pidfile)
				else:
					print(str(err))
					sys.exit(1)

	def restart(self):
		"""
		Restart the daemon
		"""
		self.stop()
		self.start()

	def run(self):
		"""
		You should override this method when you subclass Daemon. It will be called after the process has been
		daemonized by start() or restart().
		"""

		raise NotImplementedError



